/**
 * @longmo/vue2-helpers
 *
 * @description A util package to use Vue 2 with Composition API easily
 * @version 2.3.0
 * @license Apache-2.0
 * @see {@link https://github.com/longmo/vue2-helpers#readme}
 *
 * Build: 2024-11-09T16:04:27.306Z
 */
import { mapState, mapGetters, mapMutations, mapActions } from 'vuex';
import { computed, getCurrentInstance } from 'vue-demi';

const OUT_OF_SCOPE = 'getCurrentInstance() returned null. Method must be called at the top of a setup() function.';

/**
 * Vuex Helper functions
 */
function createVuexHelpers() {
    return { useState, useGetters, useMutations, useActions };
}
/**
 * Get Vuex state
 *
 * @param args - state name
 */
function useState(...args) {
    const states = mapState(...args);
    const result = {};
    Object.keys(states).forEach(key => (result[key] = computed(states[key])));
    return result;
}
/**
 * Get Vuex Getters
 *
 * @param args - getter name
 */
function useGetters(...args) {
    const getters = mapGetters(...args);
    const result = {};
    Object.keys(getters).forEach(key => (result[key] = computed(getters[key])));
    return result;
}
/**
 * Get Vue Instance
 */
function getVueInstance() {
    const vm = getCurrentInstance();
    if (vm != null) {
        return vm.proxy;
    }
    throw new Error(OUT_OF_SCOPE);
}
/**
 * Get Vuex Mutations.
 *
 * @param args - mutation name
 * @returns
 */
function useMutations(...args) {
    const vm = getVueInstance();
    const result = {};
    const mutations = mapMutations(...args);
    Object.keys(mutations).forEach(key => (result[key] = mutations[key].bind(vm)));
    return result;
}
/**
 * Get Vuex actions
 * @param args - Vuex Action
 * @returns
 */
function useActions(...args) {
    const vm = getVueInstance();
    const result = {};
    const actions = mapActions(...args);
    Object.keys(actions).forEach(key => (result[key] = actions[key].bind(vm)));
    return result;
}

export { createVuexHelpers };
//# sourceMappingURL=index.es.js.map
